#include <windows.h>

#define I8    char
#define UI8   BYTE
#define I32   int
#define UI32  DWORD
#define PI8   char *
#define PUI8  LPBYTE
#define PI32  int *
#define PUI32 LPDWORD

struct RGBINFO {

	double lfParam[12];
	DWORD dwFlg;

};

struct FILTER9INFO {

	double lfMatrix[9],lfAdd,lfDiv;

};

class SDIB32 {

public:

	SDIB32();
	SDIB32(const SDIB32*);
	SDIB32(const SDIB32&);

	SDIB32& operator=(const SDIB32&);

	bool assign(const SDIB32 *);
	bool assign(const HBITMAP);

	bool reset(I32,I32);

	void draw(HDC, I32, I32, double) const;
	void draw(HDC, I32, I32, I32,I32) const;

	I32 getWidth() const;
	I32 getHeight() const;
	I32 getColors() const;
	LPBITMAPINFO getInfo() const;
	PUI32 getPixels() const;
	PUI32 getBuffer() const;

	UI32 getRGB(I32, I32) const;
	UI8 getR(I32, I32) const;
	UI8 getG(I32, I32) const;
	UI8 getB(I32, I32) const;
	bool setRGB(I32, I32, UI32);
	bool setRGB(SDIB32*);
	bool setR(I32, I32, I32);
	bool setG(I32, I32, I32);
	bool setB(I32, I32, I32);
	UI8 getY(I32, I32) const;

	UI32 getBufferVal(I32, I32) const;
	UI8 getBuffer1(I32, I32) const;
	UI8 getBuffer2(I32, I32) const;
	UI8 getBuffer3(I32, I32) const;
	bool setBufferVal(I32, I32, UI32);
	bool setBuffer1(I32, I32, I32);
	bool setBuffer2(I32, I32, I32);
	bool setBuffer3(I32, I32, I32);

	void setRGBtoPixels();
	void setYUVtoBuffer();
	bool Filter9(FILTER9INFO*);
	void unsharp(int, double);
	bool fog(I32,double);
	bool resize(I32, I32);
	bool median();
	bool trim(I32, I32, I32, I32);
	bool mix(I32,I32, I32);
	bool trimRGB();

	~SDIB32();

private:

	I32 i32Width, i32Height;
	LPBITMAPINFO lpbiInfo;
	PUI32 lpPixels, lpBuffer;
	LPVOID lpDIB;

	void init();

};

class SBMAgent {

public:

	SBMAgent();

	~SBMAgent();

	void draw(SDIB32*, HDC, I32, I32,double lfV = 1.0);

	bool loadFile(SDIB32*, LPCTSTR);
	bool saveFile(SDIB32*, LPCTSTR,I32);
	bool loadBMP(SDIB32*, LPCTSTR);
	bool loadJpeg(SDIB32*, LPCTSTR);
	bool loadYCC(SDIB32*, LPCTSTR);
	bool loadWMF(SDIB32*, LPCTSTR);
	bool saveBMP(SDIB32*, LPCTSTR);
	bool saveJpeg(SDIB32*, LPCTSTR,I32);
	bool saveYCC(SDIB32*, LPCTSTR);

	void xorRGB(SDIB32*);

	void transR(SDIB32*, I32);
	void transG(SDIB32*, I32);
	void transB(SDIB32*, I32);
	void transY(SDIB32*, I32);
	void transU(SDIB32*, I32);
	void transV(SDIB32*, I32);

	void Filter9(SDIB32* ,FILTER9INFO *);
	void copy(SDIB32*);
	bool paste(SDIB32*);

	bool calcRGB(SDIB32*, RGBINFO*);
	void toGray(SDIB32*);

	void unsharp(SDIB32*, int, double);
	bool contY(SDIB32*, I32, I32, I32, I32);
	bool contS(SDIB32*, I32);
	bool gamma(SDIB32*, double);

	void upset(SDIB32*);
	void mirror(SDIB32*);
	void exchangeXY(SDIB32*);
	bool post(SDIB32*, I32, I32, I32);
	I32 distance(UI32, UI32);
	bool outline(SDIB32*, I32, UI32);
	bool outline2(SDIB32*, I32, UI32, UI32);
	bool aalia(SDIB32*, I32);
	bool ycut(SDIB32*, I32, I32, I32, I32, I32);
	bool rgbDef(SDIB32*, double);
	void t216(SDIB32*);

private:

	bool fJpg;

	HINSTANCE hDll;
	typedef PUI32 (CALLBACK* pLoad)(LPCTSTR);
	typedef UI32 (CALLBACK* pSave)(LPCTSTR, PUI32, UI32);
	pLoad pload;
	pSave psave;

	bool init();

};
